

#include "global.h"

#include "vgev.h"
#include "vgev_mouse.h"
#include "vgev_data.h"


/* *************************************************************************************
************************************************************************************* */
mouse_handler_type::mouse_handler_type(canvas *x) {

   reset();
   parent = x;
}


/* *************************************************************************************
************************************************************************************* */
mouse_handler_type::mouse_handler_type() {

   reset();
}


/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::reset() {
         
   parent = NULL;
   set_state(VGEV_MOUSE_MODE_RESET);
   set_state(VGEV_MOUSE_MODE_PICK);
   bear_sreadout[0] = bear_ereadout[0] = bear_rangebear[0] = 0;
   mouse_pos[0] = mouse_pos[1] = 0;
}


/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::set_state(int state) {

   if (state == VGEV_MOUSE_MODE_RESET) {
      old_mode = mouse_mode = VGEV_MOUSE_MODE_NULL;
      enable = 0;
      drawflag = 0;
      return;
   }
   
   if (state == mouse_mode)
      return;

   if (mouse_mode == VGEV_MOUSE_MODE_PICK || mouse_mode == VGEV_MOUSE_MODE_AZRANGE)
      old_mode = mouse_mode;

   mouse_mode = state;
   enable = 0;
   drawflag = 0;
}


/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::pick_platform(iface *isim, int x, int y, double lon, double lat) {

   basic_platform *ptr;
   VGEVplatform_data *qtr;
   drawable_symbol *dstr;
   dbl_llist_manager man;
   int flag = 0;
   
   if (!isim)
      return;
      
   dstr = symbol_manager.find_object(x, y, &last_pick, DRAWABLEFLAG_PLATFORM);

   qtr = dstr ? (VGEVplatform_data *)dstr->source : NULL;

   if (qtr) {
      parent->redraw_flag = -1;
      flag = 1;
      
      for (ptr=(basic_platform *)isim->head; ptr; ptr=(basic_platform *)ptr->next)
         if (ptr->vis_data)
            ((VGEVplatform_data *)ptr->vis_data)->symbol2D->set_pick(ptr->vis_data == qtr, parent);
   }

   else
      for (ptr=(basic_platform *)isim->head; ptr; ptr=(basic_platform *)ptr->next)
         if (ptr->vis_data && (((VGEVplatform_data *)ptr->vis_data)->symbol2D->flags & DRAWABLEFLAG_PICK)) {
            ((VGEVplatform_data *)ptr->vis_data)->symbol2D->set_pick(0, parent);
            parent->redraw_flag = -1;
            flag = 1;
         }


   if (flag)
      symbol_manager.resort();      
}


/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::zoom(int x, int y) {

   double pickdist2;
   double dx, dy, dblx, dbly;
   double scale;
   
   pickdist2 = MOUSE_MOVE*MOUSE_MOVE; 
   dx = x - box[0][0];
   dy = y - box[0][1];

   drawflag = 0;

   scale = parent->get_scale();
   
   // zoom out
   if (dx*dx < pickdist2 || dy*dy < pickdist2) {
      scale *= 0.5;
      parent->set_scale(&scale);
      return;
   }
   
   dblx = box[0][0] + dx * 0.5;
   dbly = box[0][1] + dy * 0.5;
   parent->recenter(&dblx, &dbly);

   dx = fabs(dx);
   dy = fabs(dy);

   scale *= (dx > dy) ? (WINX / dx) : (WINY / dy);
   parent->set_scale(&scale);
}


/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::event_motion(int x, int y, double lon, double lat) {

   char   clat[16],clon[16];
   double dx, dy, dx2, dy2;
   double angle;
   double pickdist2 = MOUSE_MOVE*MOUSE_MOVE;

   // cursor lat/lon position

   deg_to_dms(lat, lon, clat, clon, 1);
   parent->readout.stringcpy(clat);
   parent->readout.stringcat("  ");
   parent->readout.stringcat(clon);
   
   mouse_pos[0] = x;
   mouse_pos[1] = y;

   if (!enable || (mouse_mode == VGEV_MOUSE_MODE_PICK && enable != 2) || (mouse_mode != VGEV_MOUSE_MODE_ZOOM && mouse_mode != VGEV_MOUSE_MODE_AZRANGE))
      return;

   box[1][0] = x;
   box[1][1] = y;

   dx = box[0][0] - x;
   dy = box[0][1] - y;

   drawflag = dx*dx > pickdist2 || dy*dy > pickdist2;

   if (mouse_mode != VGEV_MOUSE_MODE_AZRANGE)
      return;
	   
   if (drawflag) {
      ebearlonlat[0] = lon;
      ebearlonlat[1] = lat;

      sprintf(bear_ereadout,"E: %s", parent->readout.string);

      if (!(box[0][0] - x) && !(box[0][1] - y))
         sprintf(bear_rangebear,"Azimuth: %f Range: %f NMI", 0.0f, 0.0f);
      else {
         dx = lon - sbearlonlat[0];
         dy = lat - sbearlonlat[1];
         angle = dy/sqrt(dx*dx + dy*dy);

         if (angle > 1-CORRECT)
            angle = 0;
         else if (angle < -1 + CORRECT)
            angle = 180;
         else if (dx < 0)
            angle = 360 - rad2deg(acos(angle));
         else
            angle = rad2deg(acos(angle));

         deg_to_xy(lat, lon, &dx, &dy);
         deg_to_xy(sbearlonlat[1], sbearlonlat[0], &dx2, &dy2);

         dx2 -= dx;
         dy2 -= dy;
         sprintf(bear_rangebear,"Azimuth: %f Range: %f NMI", angle, sqrt(dx2*dx2 + dy2*dy2) * MET2NMI);
      }
	 
   }

   else {
      ebearlonlat[0] = sbearlonlat[0];
      ebearlonlat[1] = sbearlonlat[1];
   }

}


/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::left_pick(int x, int y, double lon, double lat) {

   char   clat[16],clon[16];
   double pickdist2;	// should match circle symbol radius square;
   double  dx,dy;
   drawable_symbol *dstr;
   
   switch (mouse_mode) {

      case VGEV_MOUSE_MODE_RECENTER:
         return;

      case VGEV_MOUSE_MODE_PICK:

         drag_coord[0] = x;
         drag_coord[1] = y;

         pickdist2 = PICK_RADIUS*PICK_RADIUS;

         for (dstr = (drawable_symbol *)symbol_manager.head; dstr; dstr = (drawable_symbol *)dstr->next)
            if (dstr->flags & DRAWABLEFLAG_PICK)
               break;

         // see if pick is near object
         if (dstr) {
            dx = dstr->x - x;
            dy = dstr->y - y;

            if (dx*dx+dy*dy <= pickdist2) {
               enable = 2;
               return;
            }

         }

         return;

      case VGEV_MOUSE_MODE_ZOOM:

         box[0][0] = box[1][0] = x;
         box[0][1] = box[1][1] = y;
         return;

      case VGEV_MOUSE_MODE_AZRANGE:

         box[0][0] = box[1][0] = x;
         box[0][1] = box[1][1] = y;

         deg_to_dms(lat, lon, clat, clon, 1);
         sprintf(bear_sreadout,"S: %s  %s",clat,clon);

         ebearlonlat[0] = sbearlonlat[0] = lon;
         ebearlonlat[1] = sbearlonlat[1] = lat;
         return;

      default:
         return;
   }

}


/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::left_release(iface *isim, int x, int y, double lon, double lat) {

   double dx, dy;
   double pickdist2;

   switch (mouse_mode) {

      case VGEV_MOUSE_MODE_RECENTER:

         dx = x;
         dy = y;
         parent->recenter(&dx, &dy);
         set_state(old_mode);
         return;

      case VGEV_MOUSE_MODE_ZOOM:

         zoom(x, y);
         set_state(old_mode);
         return;
	 
      case VGEV_MOUSE_MODE_AZRANGE:

         pickdist2 = MOUSE_MOVE*MOUSE_MOVE; 

         dx = box[0][0] - x;
         dy = box[0][1] - y;

         if (dx*dx > pickdist2 || dy*dy > pickdist2)
            drawflag = 1;

         return;

      case VGEV_MOUSE_MODE_PICK:

         if (!isim || !enable)
            return;

         // should match circle symbol radius square;
         pickdist2 = PICK_RADIUS*PICK_RADIUS;

         dx = drag_coord[0] - x;
         dy = drag_coord[1] - y;

         // pick
         if (dx*dx+dy*dy <= pickdist2)
            pick_platform(isim, x, y, lon, lat);

         return;

      default:
         return;
   }

}


/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::event_pick(int button, int x, int y, double lon, double lat) {

   if (enable)
      return;

   enable = 1;

   switch (button) {
      case LEFTBUTTON:
         left_pick(x, y, lon, lat);
         return;
	 
      case MIDDLEBUTTON:
         return;
	 
      default: // RIGHTBUTTON
         set_state(VGEV_MOUSE_MODE_ZOOM);
         enable = 1;
         box[0][0] = box[1][0] = x;
         box[0][1] = box[1][1] = y;
         return;
   }
   
}

/* *************************************************************************************
************************************************************************************* */
void mouse_handler_type::event_release(iface *isim, int button, int x, int y, double lon, double lat) {

   double dx, dy;

   if (!enable)
      return;

   enable = 0;

   switch (button) {
      case LEFTBUTTON:
         left_release(isim, x, y, lon, lat);
         return;
	 
      case MIDDLEBUTTON:
         dx = x;
         dy = y;
         parent->recenter(&dx, &dy);
         return;
	 
      default: // RIGHTBUTTON
         zoom(x, y);
         set_state(old_mode);
         return;
   }
   
}
